@@ -1,13 +1,18 @@
 import os
 import glob
 import json
+from typing import List
 from logging import getLogger
+from PIL import Image
+import cv2
+import numpy as np
 
 import xmltodict
 
 from .exceptions import FastLabelInvalidException
 from .api import Api
-from fastlabel import converters, utils
+from fastlabel import converters, utils, const
+from fastlabel.const import AnnotationType
 
 logger = getLogger(__name__)
 
@@ -245,6 +250,39 @@ class Client:
             params["limit"] = limit
         return self.api.get_request(endpoint, params=params)
 
+    def get_video_classification_tasks(
+        self,
+        project: str,
+        status: str = None,
+        tags: list = [],
+        task_name: str = None,
+        offset: int = None,
+        limit: int = 100,
+    ) -> list:
+        """
+        Returns a list of video classification tasks.
+        Returns up to 1000 at a time, to get more, set offset as the starting position to fetch.
+
+        project is slug of your project. (Required)
+        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        tags is a list of tag. (Optional)
+        offset is the starting position number to fetch. (Optional)
+        limit is the max number to fetch. (Optional)
+        """
+        endpoint = "tasks/video/classification"
+        params = {"project": project}
+        if status:
+            params["status"] = status
+        if tags:
+            params["tags"] = tags
+        if task_name:
+            params["taskName"] = task_name
+        if offset:
+            params["offset"] = offset
+        if limit:
+            params["limit"] = limit
+        return self.api.get_request(endpoint, params=params)
+
     def get_task_id_name_map(
         self, project: str,
         offset: int = None,
@@ -368,6 +406,9 @@ class Client:
 
         endpoint = "tasks/multi-image"
         file_paths = glob.glob(os.path.join(folder_path, "*"))
+        if not file_paths:
+            raise FastLabelInvalidException(
+                "Folder does not have any file.", 422)
         contents = []
         for file_path in file_paths:
             if not utils.is_image_supported_ext(file_path):
@@ -445,6 +486,56 @@ class Client:
             payload["tags"] = tags
         return self.api.put_request(endpoint, payload=payload)
 
+    def update_image_classification_task(
+        self,
+        task_id: str,
+        status: str = None,
+        attributes: list = [],
+        tags: list = [],
+    ) -> str:
+        """
+        Create a single image classification task.
+
+        task_id is an id of the task. (Required)
+        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        attributes is a list of attribute to be set in advance. (Optional)
+        tags is a list of tag to be set in advance. (Optional)
+        """
+        endpoint = "tasks/image/classification/" + task_id
+        payload = {}
+        if status:
+            payload["status"] = status
+        if attributes:
+            payload["attributes"] = attributes
+        if tags:
+            payload["tags"] = tags
+        return self.api.put_request(endpoint, payload=payload)
+
+    def update_video_classification_task(
+        self,
+        task_id: str,
+        status: str = None,
+        attributes: list = [],
+        tags: list = [],
+    ) -> str:
+        """
+        Create a single video classification task.
+
+        task_id is an id of the task. (Required)
+        status can be 'registered', 'completed', 'skipped', 'sent_back', 'approved', 'customer_sent_back', 'customer_approved'. (Optional)
+        attributes is a list of attribute to be set in advance. (Optional)
+        tags is a list of tag to be set in advance. (Optional)
+        """
+        endpoint = "tasks/video/classification/" + task_id
+        payload = {}
+        if status:
+            payload["status"] = status
+        if attributes:
+            payload["attributes"] = attributes
+        if tags:
+            payload["tags"] = tags
+        return self.api.put_request(endpoint, payload=payload)
+
     # Task Delete
 
     def delete_task(self, task_id: str) -> None:
@@ -487,7 +578,9 @@ class Client:
                 for obj in anno["object"]:
                     f.write(obj)
                     f.write("\n")
-        with open(os.path.join(output_dir, "classes.txt"), 'w', encoding="utf8") as f:
+        classes_file_path = os.path.join(output_dir, "classes.txt")
+        os.makedirs(os.path.dirname(classes_file_path), exist_ok=True)
+        with open(classes_file_path, 'w', encoding="utf8") as f:
             for category in categories:
                 f.write(category["name"])
                 f.write("\n")
@@ -509,6 +602,134 @@ class Client:
             with open(file_path, 'w', encoding="utf8") as f:
                 f.write(xml)
 
+    def export_labelme(self, tasks: list, output_dir: str = os.path.join("output", "labelme")) -> None:
+        """
+        Convert tasks to labelme format as files.
+
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/labelme). (Optional)
+        """
+        labelmes = converters.to_labelme(tasks)
+        for labelme in labelmes:
+            file_name = labelme["imagePath"]
+            basename = utils.get_basename(file_name)
+            file_path = os.path.join(output_dir, basename + ".json")
+            os.makedirs(os.path.dirname(file_path), exist_ok=True)
+            with open(file_path, 'w') as f:
+                json.dump(labelme, f, indent=4, ensure_ascii=False)
+
+
+    # Instance / Semantic Segmetation
+    def export_instance_segmentation(self, tasks: list, output_dir: str = os.path.join("output", "instance_segmentation"), pallete: List[int] = const.COLOR_PALETTE) -> None:
+        """
+        Convert tasks to index color instance segmentation (PNG files).
+        Supports only bbox, polygon and segmentation annotation types. Hollowed points are not supported.
+        Supports up to 57 instances in default colors palette. Check const.COLOR_PALETTE for more details.
+
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/instance_segmentation). (Optional)
+        pallete is color palette of index color. Ex: [255, 0, 0, ...] (Optional)
+        """
+        tasks = converters.to_pixel_coordinates(tasks)
+        for task in tasks:
+            self.__export_index_color_image(task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=True)
+    
+    def export_semantic_segmentation(self, tasks: list, output_dir: str = os.path.join("output", "semantic_segmentation"), pallete: List[int] = const.COLOR_PALETTE) -> None:
+        """
+        Convert tasks to index color semantic segmentation (PNG files).
+        Supports only bbox, polygon and segmentation annotation types. Hollowed points are not supported.
+        Check const.COLOR_PALETTE for color pallete.
+
+        tasks is a list of tasks. (Required)
+        output_dir is output directory(default: output/semantic_segmentation). (Optional)
+        pallete is color palette of index color. Ex: [255, 0, 0, ...] (Optional)
+        """
+        classes = []
+        for task in tasks:
+            for annotation in task["annotations"]:
+                classes.append(annotation["value"])
+        classes = list(set(classes))
+        classes.sort()
+
+        tasks = converters.to_pixel_coordinates(tasks)
+        for task in tasks:
+            self.__export_index_color_image(task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=False, classes=classes)
+
+    def __export_index_color_image(self, task: list, output_dir: str, pallete: List[int], is_instance_segmentation: bool = True, classes: list = []) -> None:
+        image = Image.new("RGB", (task["width"], task["height"]), 0)
+        image = np.array(image)
+        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
+
+        index = 1
+        for annotation in task["annotations"]:
+            color = index if is_instance_segmentation else classes.index(annotation["value"]) + 1
+            if annotation["type"] == AnnotationType.segmentation.value:
+                for region in annotation["points"]:
+                    for points in region:
+                        cv_draw_points = self.__get_cv_draw_points(points)
+                        cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+                        # hollowd points are not supported
+                        break
+            elif annotation["type"] == AnnotationType.polygon.value:
+                cv_draw_points = self.__get_cv_draw_points(annotation["points"])
+                cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+            elif annotation["type"] == AnnotationType.bbox.value:
+                cv_draw_points = self.__get_cv_draw_points(annotation["points"])
+                cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+            else:
+                continue
+            index += 1
+
+        image_path = os.path.join(output_dir, utils.get_basename(task["name"]) + ".png")
+        os.makedirs(os.path.dirname(image_path), exist_ok=True)
+        image = Image.fromarray(image)
+        image = image.convert('P')
+        image.putpalette(pallete)
+        image.save(image_path)
+
+    def __get_cv_draw_points(self, points: List[int]) -> List[int]:
+        """
+        Convert points to pillow draw points. Diagonal points are not supported.
+        """
+        x_points = []
+        x_points.append(points[0])
+        x_points.append(points[1])
+        for i in range(int(len(points) / 2)):
+            if i == 0:
+                continue
+            x = points[i * 2]
+            y = points[i * 2 + 1]
+            if y > x_points[(i - 1) * 2 + 1]:
+                x_points[(i - 1) * 2] = x_points[(i - 1) * 2] - 1
+                x = x - 1
+            x_points.append(x)
+            x_points.append(y)
+
+        y_points = []
+        y_points.append(points[0])
+        y_points.append(points[1])
+        for i in range(int(len(points) / 2)):
+            if i == 0:
+                continue
+            x = points[i * 2]
+            y = points[i * 2 + 1]
+            if x < y_points[(i - 1) * 2]:
+                y_points[(i - 1) * 2 + 1] = y_points[(i - 1) * 2 + 1] - 1
+                y = y - 1
+            y_points.append(x)
+            y_points.append(y)
+
+        new_points = []
+        for i in range(int(len(points) / 2)):
+            new_points.append(x_points[i * 2])
+            new_points.append(y_points[i * 2 + 1])
+
+        cv_points = []
+        for i in range(int(len(new_points) / 2)):
+            cv_points.append((new_points[i * 2], new_points[i * 2 + 1]))
+        return np.array(cv_points)
+
+
     # Annotation
 
     def find_annotation(self, annotation_id: str) -> dict:
